=== added file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	1970-01-01 00:00:00 +0000
+++ bzrlib/bzrdir.py	2006-01-31 07:08:09 +0000
@@ -0,0 +1,322 @@
+# Copyright (C) 2005 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""BzrDir logic. The BzrDir is the basic control directory used by bzr.
+
+At format 7 this was split out into Branch, Repository and Checkout control
+directories.
+"""
+
+from copy import deepcopy
+from cStringIO import StringIO
+from unittest import TestSuite
+
+
+import bzrlib
+import bzrlib.errors as errors
+from bzrlib.lockable_files import LockableFiles
+from bzrlib.osutils import safe_unicode
+from bzrlib.trace import mutter
+from bzrlib.symbol_versioning import *
+from bzrlib.transport import get_transport
+
+
+class BzrDir(object):
+    """A .bzr control diretory.
+    
+    BzrDir instances let you create or open any of the things that can be
+    found within .bzr - checkouts, branches and repositories.
+    
+    base
+        base directory this is located under.
+    """
+
+    @staticmethod
+    def create(base):
+        """Create a new BzrDir at the url 'bzr'.
+        
+        This will call the current default formats initialize with base
+        as the only parameter.
+
+        If you need a specific format, consider creating an instance
+        of that and calling initialize().
+        """
+        return BzrDirFormat.get_default_format().initialize(safe_unicode(base))
+
+    def __init__(self, _transport, _format):
+        """Initialize a Bzr control dir object.
+        
+        Only really common logic should reside here, concrete classes should be
+        made with varying behaviours.
+
+        _format: the format that is creating this BzrDir instance.
+        _transport: the transport this dir is based at.
+        """
+        self._format = _format
+        self._transport = _transport
+
+    @staticmethod
+    def open_unsupported(base):
+        """Open a branch which is not supported."""
+        return BzrDir.open(base, _unsupported=True)
+        
+    @staticmethod
+    def open(base, _unsupported=False):
+        """Open an existing branch, rooted at 'base' (url)
+        
+        _unsupported is a private parameter to the BzrDir class.
+        """
+        t = get_transport(base)
+        mutter("trying to open %r with transport %r", base, t)
+        format = BzrDirFormat.find_format(t)
+        if not _unsupported and not format.is_supported():
+            # see open_downlevel to open legacy branches.
+            raise errors.UnsupportedFormatError(
+                    'sorry, branch format %s not supported' % format,
+                    ['use a different bzr version',
+                     'or remove the .bzr directory'
+                     ' and "bzr init" again'])
+        return format.open(t)
+
+    @staticmethod
+    def open_containing(url):
+        """Open an existing branch which contains url.
+        
+        This probes for a branch at url, and searches upwards from there.
+
+        Basically we keep looking up until we find the control directory or
+        run into the root.  If there isn't one, raises NotBranchError.
+        If there is one and it is either an unrecognised format or an unsupported 
+        format, UnknownFormatError or UnsupportedFormatError are raised.
+        If there is one, it is returned, along with the unused portion of url.
+        """
+        t = get_transport(url)
+        # this gets the normalised url back. I.e. '.' -> the full path.
+        url = t.base
+        while True:
+            try:
+                format = BzrDirFormat.find_format(t)
+                return format.open(t), t.relpath(url)
+            except errors.NotBranchError, e:
+                mutter('not a branch in: %r %s', t.base, e)
+            new_t = t.clone('..')
+            if new_t.base == t.base:
+                # reached the root, whatever that may be
+                raise errors.NotBranchError(path=url)
+            t = new_t
+
+
+class BzrDirFormat(object):
+    """An encapsulation of the initialization and open routines for a format.
+
+    Formats provide three things:
+     * An initialization routine,
+     * a format string,
+     * an open routine.
+
+    Formats are placed in an dict by their format string for reference 
+    during bzrdir opening. These should be subclasses of BzrDirFormat
+    for consistency.
+
+    Once a format is deprecated, just deprecate the initialize and open
+    methods on the format class. Do not deprecate the object, as the 
+    object will be created every system load.
+    """
+
+    _default_format = None
+    """The default format used for new branches."""
+
+    _formats = {}
+    """The known formats."""
+
+    @classmethod
+    def find_format(klass, transport):
+        """Return the format registered for URL."""
+        try:
+            format_string = transport.get(".bzr/branch-format").read()
+            return klass._formats[format_string]
+        except errors.NoSuchFile:
+            raise errors.NotBranchError(path=transport.base)
+        except KeyError:
+            raise errors.UnknownFormatError(format_string)
+
+    @classmethod
+    def get_default_format(klass):
+        """Return the current default format."""
+        return klass._default_format
+
+    def get_format_string(self):
+        """Return the ASCII format string that identifies this format."""
+        raise NotImplementedError(self.get_format_string)
+
+    def initialize(self, url):
+        """Create a bzr control dir at this url and return an opened copy."""
+        # Since we don't have a .bzr directory, inherit the
+        # mode from the root directory
+        t = get_transport(url)
+        temp_control = LockableFiles(t, '')
+        temp_control._transport.mkdir('.bzr',
+                                      # FIXME: RBC 20060121 dont peek under
+                                      # the covers
+                                      mode=temp_control._dir_mode)
+        file_mode = temp_control._file_mode
+        del temp_control
+        mutter('created control directory in ' + t.base)
+        control = t.clone('.bzr')
+        lock_file = 'branch-lock'
+        utf8_files = [('README', 
+                       "This is a Bazaar-NG control directory.\n"
+                       "Do not change any files in this directory.\n"),
+                      ('branch-format', self.get_format_string()),
+                      ]
+        # NB: no need to escape relative paths that are url safe.
+        control.put(lock_file, StringIO(), mode=file_mode)
+        control_files = LockableFiles(control, lock_file)
+        control_files.lock_write()
+        try:
+            for file, content in utf8_files:
+                control_files.put_utf8(file, content)
+        finally:
+            control_files.unlock()
+        return BzrDir(t, self)
+
+    def is_supported(self):
+        """Is this format supported?
+
+        Supported formats must be initializable and openable.
+        Unsupported formats may not support initialization or committing or 
+        some other features depending on the reason for not being supported.
+        """
+        return True
+
+    def open(self, transport, _found=False):
+        """Return an instance of this format for the dir transport points at.
+        
+        _found is a private parameter, do not use it.
+        """
+        if not _found:
+            assert isinstance(BzrDirFormat.find_format(transport),
+                              self.__class__)
+        return BzrDir(transport, self)
+
+    @classmethod
+    def register_format(klass, format):
+        klass._formats[format.get_format_string()] = format
+
+    @classmethod
+    def set_default_format(klass, format):
+        klass._default_format = format
+
+    @classmethod
+    def unregister_format(klass, format):
+        assert klass._formats[format.get_format_string()] is format
+        del klass._formats[format.get_format_string()]
+
+
+class BzrDirFormat4(BzrDirFormat):
+    """Bzr dir format 4.
+
+    This format is a combined format for working tree, branch and repository.
+    It has:
+     - flat stores
+     - TextStores for texts, inventories,revisions.
+
+    This format is deprecated: it indexes texts using a text it which is
+    removed in format 5; write support for this format has been removed.
+    """
+
+    def get_format_string(self):
+        """See BzrDirFormat.get_format_string()."""
+        return "Bazaar-NG branch, format 0.0.4\n"
+
+    def initialize(self, url):
+        """Format 4 branches cannot be created."""
+        raise errors.UninitializableFormat(self)
+
+    def is_supported(self):
+        """Format 4 is not supported.
+
+        It is not supported because the model changed from 4 to 5 and the
+        conversion logic is expensive - so doing it on the fly was not 
+        feasible.
+        """
+        return False
+
+
+class BzrDirFormat5(BzrDirFormat):
+    """Bzr control format 5.
+
+    This format is a combined format for working tree, branch and repository.
+    It has:
+     - weaves for file texts and inventory
+     - flat stores
+     - TextStores for revisions and signatures.
+    """
+
+    def get_format_string(self):
+        """See BzrDirFormat.get_format_string()."""
+        return "Bazaar-NG branch, format 5\n"
+
+
+class BzrDirFormat6(BzrDirFormat):
+    """Bzr control format 6.
+
+    This format is a combined format for working tree, branch and repository.
+    It has:
+     - weaves for file texts and inventory
+     - hash subdirectory based stores.
+     - TextStores for revisions and signatures.
+    """
+
+    def get_format_string(self):
+        """See BzrDirFormat.get_format_string()."""
+        return "Bazaar-NG branch, format 6\n"
+
+
+BzrDirFormat.register_format(BzrDirFormat4())
+BzrDirFormat.register_format(BzrDirFormat5())
+__default_format = BzrDirFormat6()
+BzrDirFormat.register_format(__default_format)
+BzrDirFormat.set_default_format(__default_format)
+
+
+class BzrDirTestProviderAdapter(object):
+    """A tool to generate a suite testing multiple bzrdir formats at once.
+
+    This is done by copying the test once for each transport and injecting
+    the transport_server, transport_readonly_server, and bzrdir_format
+    classes into each copy. Each copy is also given a new id() to make it
+    easy to identify.
+    """
+
+    def __init__(self, transport_server, transport_readonly_server, formats):
+        self._transport_server = transport_server
+        self._transport_readonly_server = transport_readonly_server
+        self._formats = formats
+    
+    def adapt(self, test):
+        result = TestSuite()
+        for format in self._formats:
+            new_test = deepcopy(test)
+            new_test.transport_server = self._transport_server
+            new_test.transport_readonly_server = self._transport_readonly_server
+            new_test.bzrdir_format = format
+            def make_new_test_id():
+                new_id = "%s(%s)" % (new_test.id(), format.__class__.__name__)
+                return lambda: new_id
+            new_test.id = make_new_test_id()
+            result.addTest(new_test)
+        return result

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2006-01-30 01:05:45 +0000
+++ bzrlib/tests/__init__.py	2006-01-31 07:08:09 +0000
@@ -81,10 +81,12 @@
     import bzrlib.doc
     import bzrlib.tests.blackbox
     import bzrlib.tests.branch_implementations
+    import bzrlib.tests.bzrdir_implementations
     return [
             bzrlib.doc,
             bzrlib.tests.blackbox,
             bzrlib.tests.branch_implementations,
+            bzrlib.tests.bzrdir_implementations,
             ]
 
 
@@ -833,6 +835,7 @@
                    'bzrlib.tests.test_bad_files',
                    'bzrlib.tests.test_basis_inventory',
                    'bzrlib.tests.test_branch',
+                   'bzrlib.tests.test_bzrdir',
                    'bzrlib.tests.test_command',
                    'bzrlib.tests.test_commit',
                    'bzrlib.tests.test_commit_merge',

=== added directory 'bzrlib/tests/bzrdir_implementations'
=== added file 'bzrlib/tests/bzrdir_implementations/__init__.py'
--- bzrlib/tests/bzrdir_implementations/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/bzrdir_implementations/__init__.py	2006-01-31 07:08:09 +0000
@@ -0,0 +1,49 @@
+# Copyright (C) 2006 by Canonical Ltd
+# Authors: Robert Collins <robert.collins@canonical.com>
+# -*- coding: utf-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""BzrDir implementation tests for bzr.
+
+These test the conformance of all the bzrdir variations to the expected API.
+Specific tests for individual formats are in the tests/test_bzrdir.py file 
+rather than in tests/branch_implementations/*.py.
+"""
+
+from bzrlib.bzrdir import BzrDirTestProviderAdapter, BzrDirFormat
+from bzrlib.tests import (
+                          adapt_modules,
+                          default_transport,
+                          TestLoader,
+                          TestSuite,
+                          )
+
+
+def test_suite():
+    result = TestSuite()
+    test_bzrdir_implementations = [
+        'bzrlib.tests.bzrdir_implementations.test_bzrdir',
+        ]
+    adapter = BzrDirTestProviderAdapter(
+        default_transport,
+        # None here will cause a readonly decorator to be created
+        # by the TestCaseWithTransport.get_readonly_transport method.
+        None,
+        BzrDirFormat._formats.values())
+    loader = TestLoader()
+    adapt_modules(test_bzrdir_implementations, adapter, loader, result)
+    return result

=== added file 'bzrlib/tests/bzrdir_implementations/test_bzrdir.py'
--- bzrlib/tests/bzrdir_implementations/test_bzrdir.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/bzrdir_implementations/test_bzrdir.py	2006-01-31 07:08:09 +0000
@@ -0,0 +1,129 @@
+# (C) 2005, 2006 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for bzrdir implementations - tests a bzrdir format."""
+
+import os
+import sys
+
+import bzrlib.branch as branch
+import bzrlib.bzrdir as bzrdir
+from bzrlib.branch import Branch, needs_read_lock, needs_write_lock
+from bzrlib.commit import commit
+import bzrlib.errors as errors
+from bzrlib.errors import (FileExists,
+                           NoSuchRevision,
+                           NoSuchFile,
+                           UninitializableFormat,
+                           NotBranchError,
+                           )
+import bzrlib.gpg
+from bzrlib.osutils import getcwd
+from bzrlib.revision import NULL_REVISION
+from bzrlib.tests import TestCase, TestCaseWithTransport, TestSkipped
+from bzrlib.trace import mutter
+import bzrlib.transactions as transactions
+from bzrlib.transport import get_transport
+from bzrlib.upgrade import upgrade
+from bzrlib.workingtree import WorkingTree
+
+
+class TestCaseWithBzrDir(TestCaseWithTransport):
+
+    def setUp(self):
+        super(TestCaseWithBzrDir, self).setUp()
+        self.bzrdir = None
+
+    def get_bzrdir(self):
+        if self.bzrdir is None:
+            self.bzrdir = self.make_bzrdir(None)
+        return self.bzrdir
+
+    def make_bzrdir(self, relpath):
+        try:
+            url = self.get_url(relpath)
+            segments = url.split('/')
+            if segments and segments[-1] not in ('', '.'):
+                parent = '/'.join(segments[:-1])
+                t = get_transport(parent)
+                try:
+                    t.mkdir(segments[-1])
+                except FileExists:
+                    pass
+            return self.bzrdir_format.initialize(url)
+        except UninitializableFormat:
+            raise TestSkipped("Format %s is not initializable.")
+
+
+class TestBzrDir(TestCaseWithBzrDir):
+
+    def test_clone_bzrdir(self):
+        #TODO: Test that we can create a clone of a bzr dir 
+        #
+        #... and all its contents verbatim.
+        raise AssertionError('not tested yet.')
+
+        # a bunch of tests needed::
+        # create a bzr dir with nothing, clone it check result has nothing
+        # create a bzr dir with storage only, clone it check result has same
+        # storage contents
+        # create bzr dir with branch with no storage, clone, check resulting
+        # dir also has no storage but does have branch
+        # create bzr dir with a tree but no storage or branch (on local disk
+        # as thats how workingtree works) clone and check no branch or 
+        # storage is made
+        # create a standalone_branch , clone that and check all bits are
+        # clone.
+        # features that are not supported (like detached working trees for
+        # older formats, should catch the error and silently pass the test 
+        # as they are honouring the format.
+
+    def test_new_line_of_development_bzrdir(self):
+        #TODO: Test that we can create a line of development from a 
+        raise AssertionError('not tested yet.')
+         
+        # same permutations as clone_bzrdir, or nearly so, but we want to
+        # have checkouts force creation of a new branch because thats the 
+        # desired semantic.
+
+
+    def test_format_initialize_find_open(self):
+        # loopback test to check the current format initializes to itself.
+        if not self.bzrdir_format.is_supported():
+            # unsupported formats are not loopback testable
+            # because the default open will not open them and
+            # they may not be initializable.
+            return
+        # supported formats must be able to init and open
+        t = get_transport(self.get_url())
+        readonly_t = get_transport(self.get_readonly_url())
+        made_control = self.bzrdir_format.initialize(t.base)
+        self.failUnless(isinstance(made_control, bzrdir.BzrDir))
+        self.assertEqual(self.bzrdir_format,
+                         bzrdir.BzrDirFormat.find_format(readonly_t))
+        direct_opened_dir = self.bzrdir_format.open(readonly_t)
+        opened_dir = bzrdir.BzrDir.open(t.base)
+        self.assertEqual(made_control._format,
+                         opened_dir._format)
+        self.assertEqual(direct_opened_dir._format,
+                         opened_dir._format)
+        self.failUnless(isinstance(opened_dir, bzrdir.BzrDir))
+
+    def test_open_not_bzrdir(self):
+        # test the formats specific behaviour for no-content or similar dirs.
+        self.assertRaises(NotBranchError,
+                          self.bzrdir_format.open,
+                          get_transport(self.get_readonly_url()))

=== modified file 'bzrlib/tests/test_branch.py'
--- bzrlib/tests/test_branch.py	2006-01-24 04:44:33 +0000
+++ bzrlib/tests/test_branch.py	2006-01-31 07:08:09 +0000
@@ -16,7 +16,7 @@
 
 """Tests for the Branch facility that are not interface  tests.
 
-For interface tests see test_branch_implementations.py.
+For interface tests see tests/branch_implementations/*.py.
 
 For concrete class tests see this file, and for meta-branch tests
 also see this file.

=== added file 'bzrlib/tests/test_bzrdir.py'
--- bzrlib/tests/test_bzrdir.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_bzrdir.py	2006-01-31 07:08:09 +0000
@@ -0,0 +1,147 @@
+# (C) 2005 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for the BzrDir facility and any format specific tests.
+
+For interface contract tests, see tests/bzr_dir_implementations.
+"""
+
+from StringIO import StringIO
+
+import bzrlib.bzrdir as bzrdir
+from bzrlib.errors import (NotBranchError,
+                           UnknownFormatError,
+                           UnsupportedFormatError,
+                           )
+
+from bzrlib.tests import TestCase, TestCaseWithTransport
+from bzrlib.transport import get_transport
+from bzrlib.transport.http import HttpServer
+from bzrlib.transport.memory import MemoryServer
+
+
+class TestDefaultFormat(TestCase):
+
+    def test_get_set_default_initializer(self):
+        old_format = bzrdir.BzrDirFormat.get_default_format()
+        # default is BzrDirFormat6
+        self.failUnless(isinstance(old_format, bzrdir.BzrDirFormat6))
+        bzrdir.BzrDirFormat.set_default_format(SampleBzrDirFormat())
+        # creating a bzr dir should now create an instrumented dir.
+        try:
+            result = bzrdir.BzrDir.create('memory:/')
+            self.assertEqual(result,  'A bzr dir')
+        finally:
+            bzrdir.BzrDirFormat.set_default_format(old_format)
+        self.assertEqual(old_format, bzrdir.BzrDirFormat.get_default_format())
+
+
+class SampleBzrDirFormat(bzrdir.BzrDirFormat):
+    """A sample format
+
+    this format is initializable, unsupported to aid in testing the 
+    open and open_downlevel routines.
+    """
+
+    def get_format_string(self):
+        """See BzrDirFormat.get_format_string()."""
+        return "Sample .bzr dir format."
+
+    def initialize(self, url):
+        """Create a bzr dir."""
+        t = get_transport(url)
+        t.mkdir('.bzr')
+        t.put('.bzr/branch-format', StringIO(self.get_format_string()))
+        return 'A bzr dir'
+
+    def is_supported(self):
+        return False
+
+    def open(self, transport):
+        return "opened branch."
+
+
+class TestBzrDirFormat(TestCaseWithTransport):
+    """Tests for the BzrDirFormat facility."""
+
+    def test_find_format(self):
+        # is the right format object found for a branch?
+        # create a branch with a few known format objects.
+        # this is not quite the same as 
+        t = get_transport(self.get_url())
+        self.build_tree(["foo/", "bar/"], transport=t)
+        def check_format(format, url):
+            format.initialize(url)
+            t = get_transport(url)
+            found_format = bzrdir.BzrDirFormat.find_format(t)
+            self.failUnless(isinstance(found_format, format.__class__))
+        check_format(bzrdir.BzrDirFormat5(), "foo")
+        check_format(bzrdir.BzrDirFormat6(), "bar")
+        
+    def test_find_format_nothing_there(self):
+        self.assertRaises(NotBranchError,
+                          bzrdir.BzrDirFormat.find_format,
+                          get_transport('.'))
+
+    def test_find_format_unknown_format(self):
+        t = get_transport(self.get_url())
+        t.mkdir('.bzr')
+        t.put('.bzr/branch-format', StringIO())
+        self.assertRaises(UnknownFormatError,
+                          bzrdir.BzrDirFormat.find_format,
+                          get_transport('.'))
+
+    def test_register_unregister_format(self):
+        format = SampleBzrDirFormat()
+        url = self.get_url()
+        # make a bzrdir
+        format.initialize(url)
+        # register a format for it.
+        bzrdir.BzrDirFormat.register_format(format)
+        # which bzrdir.Open will refuse (not supported)
+        self.assertRaises(UnsupportedFormatError, bzrdir.BzrDir.open, url)
+        # but open_downlevel will work
+        t = get_transport(url)
+        self.assertEqual(format.open(t), bzrdir.BzrDir.open_unsupported(url))
+        # unregister the format
+        bzrdir.BzrDirFormat.unregister_format(format)
+        # now open_downlevel should fail too.
+        self.assertRaises(UnknownFormatError, bzrdir.BzrDir.open_unsupported, url)
+
+
+class ChrootedTests(TestCaseWithTransport):
+    """A support class that provides readonly urls outside the local namespace.
+
+    This is done by checking if self.transport_server is a MemoryServer. if it
+    is then we are chrooted already, if it is not then an HttpServer is used
+    for readonly urls.
+    """
+
+    def setUp(self):
+        super(ChrootedTests, self).setUp()
+        if not self.transport_server == MemoryServer:
+            self.transport_readonly_server = HttpServer
+
+    def test_open_containing(self):
+        self.assertRaises(NotBranchError, bzrdir.BzrDir.open_containing,
+                          self.get_readonly_url(''))
+        self.assertRaises(NotBranchError, bzrdir.BzrDir.open_containing,
+                          self.get_readonly_url('g/p/q'))
+        control = bzrdir.BzrDir.create(self.get_url())
+        branch, relpath = bzrdir.BzrDir.open_containing(self.get_readonly_url(''))
+        self.assertEqual('', relpath)
+        branch, relpath = bzrdir.BzrDir.open_containing(self.get_readonly_url('g/p/q'))
+        self.assertEqual('g/p/q', relpath)

=== modified file 'bzrlib/tests/test_selftest.py'
--- bzrlib/tests/test_selftest.py	2006-01-26 22:30:47 +0000
+++ bzrlib/tests/test_selftest.py	2006-01-31 07:08:09 +0000
@@ -230,6 +230,30 @@
         self.assertEqual(tests[1].transport_readonly_server, server2)
 
 
+class TestBzrDirProviderAdapter(TestCase):
+    """A group of tests that test the bzr dir implementation test adapter."""
+
+    def test_adapted_tests(self):
+        # check that constructor parameters are passed through to the adapted
+        # test.
+        from bzrlib.bzrdir import BzrDirTestProviderAdapter
+        input_test = TestBzrDirProviderAdapter(
+            "test_adapted_tests")
+        server1 = "a"
+        server2 = "b"
+        formats = ["c", "d"]
+        adapter = BzrDirTestProviderAdapter(server1, server2, formats)
+        suite = adapter.adapt(input_test)
+        tests = list(iter(suite))
+        self.assertEqual(2, len(tests))
+        self.assertEqual(tests[0].bzrdir_format, formats[0])
+        self.assertEqual(tests[0].transport_server, server1)
+        self.assertEqual(tests[0].transport_readonly_server, server2)
+        self.assertEqual(tests[1].bzrdir_format, formats[1])
+        self.assertEqual(tests[1].transport_server, server1)
+        self.assertEqual(tests[1].transport_readonly_server, server2)
+
+
 class TestTestCaseWithTransport(TestCaseWithTransport):
     """Tests for the convenience functions TestCaseWithTransport introduces."""
 

=== modified file 'bzrlib/transport/__init__.py'
--- bzrlib/transport/__init__.py	2006-01-26 20:52:58 +0000
+++ bzrlib/transport/__init__.py	2006-01-31 07:08:09 +0000
@@ -603,5 +603,5 @@
 register_lazy_transport('https://', 'bzrlib.transport.http', 'HttpTransport')
 register_lazy_transport('ftp://', 'bzrlib.transport.ftp', 'FtpTransport')
 register_lazy_transport('aftp://', 'bzrlib.transport.ftp', 'FtpTransport')
-register_lazy_transport('memory://', 'bzrlib.transport.memory', 'MemoryTransport')
+register_lazy_transport('memory:/', 'bzrlib.transport.memory', 'MemoryTransport')
 register_lazy_transport('readonly+', 'bzrlib.transport.readonly', 'ReadonlyTransportDecorator')

